/*
 * Code Mercenaries Hard- und Software GmbH
 * Karl-Marx-Strasse 147A
 * 12529 Schönefeld
 * Deutschland
 * 
 * August 2015
 * 
 * Used result values:
 *   -1 hard error
 *       1 light weight error (like wrong user input)
 *    0 success
 * 
 * This program is using the BCM2835 library for 
 * Raspberry Pi 2. More information at
 * http://www.airspayce.com/mikem/bcm2835/
 * */
 
#include <stdio.h>
#include <string.h>
#include <bcm2835.h>
#include <stdlib.h>
#include "led_warrior04.h"

char  readArguments(int, char**);
void  printHelp();

/*
 * Read in the command line arguments
 * */
char readArguments(int argc, char** argv) {
    
    int buf;
    char mode;
    int reg;
    char result;
    
    //search for help command
    for(int i = 1; i<argc; i++) {
        
        if(strstr(argv[i], "help")) {
            printHelp();
            return 0;
        }
    }
    //started program without parameter
    if(argc == 1) {
        printHelp();
        return 0;
    }
    
    if((argc >= 2) && strcmp(argv[1], "-l") == 0) {
        result = LW04_listDevices();
        if(result != 0) return -1;
        return 0;
    }
    
    //get address for the I2C device
    if((argc >= 3) && (strcmp(argv[1], "-a") == 0)){
        buf = atoi(argv[2]);
        if(buf >= 0 && buf <= 127) {
            LW04_setSlave(buf);
        }else {
            printf("Invalid address! Address has to be between 0 and 127\r\n");
            return 1;
        }
    }else {
        printf("Destination address not given!\r\n");
        return 1;
    }
    
    //strings for readInfo()
    char serialNumber[30] = {0};
    char version[30]      = {0};
    char model[30]        = {0};
    
    //for getColorOutput and getCurrent
    int value;
    
    //i = 3 to start by "real" parameter (argv[0] is the command for starting this program and -a is obligatory)
    for(int i = 3; i<argc; i++) {
        
        if(argv[i][0] == '-' && strlen(argv[i])>1) {
            
            //get parameter
            switch(argv[i][1]) {
                
                //test device
                case 't':   result = LW04_testDevice();
                            break;
                           
                //info about the device
                case 'i':   result = LW04_readInfo(serialNumber, version, model);
                            printf("%s\r\n%s\r\n%s\r\n", serialNumber, version, model);
                            break;
                         
                //disable channel
                case 'd':   do {
                                if(++i>=argc) return 1;
                                switch(argv[i][0]) {
                                    
                                    case 'r':   result = LW04_disableChannel(0);
                                                break;
                                                
                                    case 'g':   result = LW04_disableChannel(1);
                                                break;
                                                
                                    case 'b':   result = LW04_disableChannel(2);
                                                break;
                                                
                                    case 'w':   result = LW04_disableChannel(3);
                                                break;
                                                
                                    default:    return 1;
                                }
                                if(result != 0) return -1;
                                
                            //next argument for this command or next command?
                            }while((i+1 < argc) && (argv[i+1][0] != '-'));
                            
                            break;
                            
                //enable channel
                case 'e':   do {
                                if(++i>=argc) return 1;
                                switch(argv[i][0]) {
                                    
                                    case 'r':   result = LW04_enableChannel(0);
                                                break;
                                                
                                    case 'g':   result = LW04_enableChannel(1);
                                                break;
                                                
                                    case 'b':   result = LW04_enableChannel(2);
                                                break;
                                                
                                    case 'w':   result = LW04_enableChannel(3);
                                                break;
                                                
                                    default:    return 1;
                                }
                                if(result != 0) return -1;
                            
                            //next argument for this command or next command?
                            }while((i+1 < argc) && (argv[i+1][0] != '-'));
                            
                            break;
                            
                //setting current needs three arguments: read/write mode, channel and value
                case 'c':   if(i+2 >= argc) return 1;
                
                            //first argument
                            switch(argv[++i][0]) {
                                
                                case 'W':   mode = LW04_WRITE;
                                            break;
                                            
                                case 'R':   mode = LW04_READ;
                                            break;
                                            
                                default:    return 1;
                            }
                            
                            //write needs one argument more than read
                            if(mode == LW04_WRITE && i+2 >= argc) return 1;
                            
                            do {
                                //second argument
                                switch(argv[i+1][0]) {
                                    
                                    case 'r':   reg = LW04_REG_CURRENT_1;
                                                break;
                                                
                                    case 'g':   reg = LW04_REG_CURRENT_2;
                                                break;
                                                
                                    case 'b':   reg = LW04_REG_CURRENT_3;
                                                break;
                                                
                                    case 'w':   reg = LW04_REG_CURRENT_4;
                                                break;
                                                
                                    default:    return 1;
                                }
                                
                                //do work
                                if(mode == LW04_WRITE) {
                                    //third argument
                                    buf = atoi(argv[i+2]);
                                    if(buf>100 && buf<0) return 1;
                                        
                                    result = LW04_setCurrent(reg, buf);
                                    if(result != 0) return -1;
                                    
                                    i = i+2;
                                    
                                }else if(mode == LW04_READ) {
                                    result = LW04_getCurrent(reg, &value);
                                    if(result != 0) return -1;
                                    
                                    printf("Register %d has the value: %d mA\n", reg, value);
                                    
                                    i = i+1;
                                }
                                
                            //next argument for this command or next command?
                            }while((i+1 < argc) && (argv[i+1][0] != '-'));
                            
                            break;
                          
                //set dimm mode
                case 's':   if(++i>=argc) return 1;
                            buf = atoi(argv[i]);
                            result = LW04_dimmer(buf);
                            break;
                
                //set color brightness
                case 'f':   if(i+2 >= argc) return 1;
                            
                            //first argument
                            switch(argv[++i][0]) {
                                
                                case 'W':   mode = LW04_WRITE;
                                            break;
                                            
                                case 'R':   mode = LW04_READ;
                                            break;
                                            
                                default:    return 1;
                            }
                            
                            //write needs one argument more than read
                            if(mode == LW04_WRITE && i+2 >= argc) return 1;
                            
                            do {
                                //second argument
                                switch(argv[i+1][0]) {
                                    
                                    case 'r':   reg = LW04_REG_OUTPUT_1;
                                                break;
                                                
                                    case 'g':   reg = LW04_REG_OUTPUT_2;
                                                break;
                                                
                                    case 'b':   reg = LW04_REG_OUTPUT_3;
                                                break;
                                                
                                    case 'w':   reg = LW04_REG_OUTPUT_4;
                                                break;
                                                
                                    default:    return 1;
                                }
                                
                                //do work
                                if(mode == LW04_WRITE) {
                                    //third argument
                                    buf = atoi(argv[i+2]);
                                    if(buf>100 && buf<0) {
                                        printf("Color values have to be between 0 and 100!\r\n");
                                        printf("Invalid number %d\r\n", buf);
                                        return 1;
                                    }
                                    result = LW04_setColorOutput(reg, LW04_convertColor(buf));
                                    if(result != 0) return -1;
                                    
                                    i = i+2;
                                    
                                }else if(mode == LW04_READ) {
                                    result = LW04_getColorOutput(reg, &value);
                                    if(result != 0) return -1;
                                    
                                    printf("Register %d has the value: %d\n", reg, value);
                                    
                                    i = i+1;
                                }
                                
                            //next argument for this command or next command?
                            }while((i+1 < argc) && (argv[i+1][0] != '-'));
                            
                            break;
                      
                //save specific data wich should be restored after device is plugged off and on again
                case 'p':   if(i+1 >= argc) return 1;
                            buf = atoi(argv[++i]);
                            result = LW04_saveDataPermanent((char)buf);
                            break;
                            
                //list all possible devices with addresses
                case 'l':   result = LW04_listDevices();
                            break;
                            
                //no valid parameter, show help
                default:    printHelp();
                            return 0;
                            
            }
            if(result != 0) return -1;
            
        }else {
            return 1;
        }
        
    }
    
    printf("Command accepted\r\n");
    
    return 0;
}

void printHelp() {
    
    printf("Usage:\r\n");
    printf("-l List all active device addresses. Has to be first argument and all other arguments are ignored\r\n\r\n");
    printf("-a x Has to be the first parameter if not using \"-l\" (x from 0 to 127)\r\n\r\n");
    printf("-t Test the device\r\n\r\n");
    printf("-i Info about the device\r\n\r\n");
    printf("-d [x1 x2...xn] Disable channel x1...xn (x from {r, g, b, w})\r\n\r\n");
    printf("-e [x1 x2...xn] Enable channel x1...xn (x from {r, g, b, w})\r\n\r\n");
    printf("-c W [x1 x2...xn] [y1 y2...yn] Set current of channel x1...xn (x from {r, g, b, w}; y value in mA)\r\n");
    printf("   R [x1 x2...xn]              Get current of channel x1...xn (x from {r, g, b, w})\r\n\r\n");
    printf("-f W [x1 x2...xn] [y1 y2...yn] Set brightness of channel x1...xn (x from {r, g, b, w}; y from 0 to 100)\r\n");
    printf("   R [x1 x2...xn]              Get brightness of channel x1...xn (x from {r, g, b, w})\r\n\r\n");
    printf("-s [0...3] mode one of {0 (I2C), 1 (DMX512), 2 (DALI), 3 (Sequence Mode)}\r\n\r\n");
    printf("-p x Save values permanent in device\r\n");
    printf("     (x from {1 (Moderegister 0 and 1), 2 (Dimming values), 4 (Current settings)}\r\n");
    printf("     or a sum of them for specific values wich should be saved permanent.\r\n");
    printf("     For example for saving Current and Dimmer the needed value is 2+4, so enter \"-p 6\")\r\n\r\n");
    printf("r    red channel\r\n");
    printf("g    green channel\r\n");
    printf("b    blue channel\r\n");
    printf("w    white channel\r\n\r\n");
}

int main(int argc, char** argv) {

    //initialize library for I2C
    if(!bcm2835_init()) {
        printf("No initialization possible! BCM2835 library needs root access, so try running with sudo\r\n");
        return 1;
    }
    //begin connection
    bcm2835_i2c_begin();
    bcm2835_i2c_setSlaveAddress(0);
    bcm2835_i2c_setClockDivider(BCM2835_I2C_CLOCK_DIVIDER_2500);
    
    signed char result = readArguments(argc, argv);
    
    if(result == 1) {
        printf("Error by reading in the arguments!\r\n");
        return 1;
        
    }else if(result == -1) {
        printf("Internal error\r\n");
        return 1;
    }
    
    //end connection
    bcm2835_i2c_end();
    
    return 0;
}
